import json
import csv
import abc

class AbstractLootTable(object):
    __metaclass__ = abc.ABCMeta

    def to_list(self):
        raise NotImplementedError

    def to_repr(self):
        raise NotImplementedError

    def to_intermediate(self):
        raise NotImplementedError


class FileLootTable(AbstractLootTable):
    def __init__(self, path_to_file):
        self.path_to_file = path_to_file


class ListLootTable(AbstractLootTable):
    def __init__(self, loot_list):
        self.loot_list = loot_list

    def to_list(self):
        return self.loot_list

    def to_repr(self):
        return str(self.to_list())

    def to_intermediate(self):
        return self.loot_list


class JSONLootTable(FileLootTable):
    def to_intermediate(self):
        if hasattr(self, 'intermediate_result'):
            return self.intermediate_result
        loot_file = open(self.path_to_file, 'r')
        intermediate_result = json.load(loot_file)
        loot_file.close()
        self.intermediate_result = intermediate_result
        return intermediate_result

    def to_repr(self):
        return json.dumps(self.to_intermediate(), indent=4)

    def to_list(self):
        return self.to_intermediate()


class CSVLootTable(FileLootTable):
    def to_intermediate(self):
        if hasattr(self, 'intermediate_result'):
            return self.intermediate_result
        loot_file = open(self.path_to_file, 'r')
        row_parser = csv.reader(loot_file, quotechar='"', delimiter=',')
        intermediate_result = []
        keys_list = []
        for index, raw_row in enumerate(row_parser):            
            row = map(lambda s: s.strip(), raw_row)
            row_raw_values = [cell for cell in row]
            if index == 0:
                keys_list = row_raw_values
            else:
                prepared_row = map(self._map_to_float, row_raw_values)
                row_dict = zip(keys_list, prepared_row)
                intermediate_result.append(row_dict)
        self.intermediate_result = intermediate_result
        loot_file.close()
        return intermediate_result

    def _map_to_float(self, item):
        try:
            return float(item)
        except ValueError:
            return item

    def to_repr(self):
        return json.dumps(self.to_intermediate(), indent=4)

    def to_list(self):
        return self.to_intermediate()



import pprint
import random

class AbstractLootGenerator(object):
    RESULT_NAME = None
    VALUE_NAME = None
    EXTRA_NAME = None

    def __init__(self, loot_table, result_file_name=None, *args,  **kwargs):
        self.loot_table = loot_table
        self.result_file_name = result_file_name
        self.selection_options = loot_table.as_list()

    def generate(self, amount, *args, **kwargs):
        result_dict = {}
        generateable = True
        while generateable:
            generation_options = self.select_for_generation(amount, *args, **kwargs)
            generateable = bool(generation_options)
            if generateable:
                result = self.select_loot(generation_options)
                amount = amount - result[self.VALUE_NAME]
                if result[self.RESULT_NAME] in result_dict.keys():
                    result_dict[self.RESULT_NAME] += 1
                else:
                    result_dict[self.RESULT_NAME] = 1
            else:
                result_dict[self.EXTRA_NAME] = amount
        self.print_result(result_dict)

    def select_for_generation(self, amount,  *args, **kwargs):
        raise NotImplementedError

    def select_loot(self, generation_options):
        raise NotImplementedError

    def print_result(self, result):
        if self.result_file_name:
            result_file = open(self.result_file_name, 'w')
            printer = pprint.PrettyPrinter(indent=4, stream=result_file)
        else:
            printer = pprint.PrettyPrinter(indent=4)
        printer.pprint(result)
        if self.result_file_name:
            result_file.close()


class BasicLootGenerator(AbstractLootGenerator):
    RESULT_NAME = 'item'
    VALUE_NAME = 'value'
    EXTRA_NAME = 'extra'

    def select_loot(self, generation_options):
        return random.choice(generation_options)
    
    def select_for_generation(self, amount, *args, **kwargs):
        filtered_selection_options = filter(lambda x: x[self.VALUE_NAME] <= amount, self.selection_options)
        return filtered_selection_options


class MiscLootGenerator(BasicLootGenerator):
    MISC_KWARG = 'exclude_misc'
    MISC_NAME = 'is_misc'

    def select_for_generation(self, amount, *args, **kwargs):
        filtered_selection_options = super(MiscLootGenerator, self).select_for_generation(amount, *args, **kwargs)
        if self.MISC_KWARG in kwargs:
            filtered_selection_options = filter(lambda x: x[self.MISC_NAME], filtered_selection_options)
        return filtered_selection_options



class WeightedLootGenerator(MiscLootGenerator):
    WEIGHT_NAME = 'weight'

    def select_loot(self, generation_options):
        weights_list = [item[self.WEIGHT_NAME] for item in generation_options]
        return generation_options[self.weighted_choice_sub(weights_list)]

    def weighted_choice_sub(self, weights):
        rnd = random.random() * sum(weights)
        for i, w in enumerate(weights):
            rnd -= w
            if rnd < 0:
                return i



from setuptools import setup, find_packages


def readme():
    with open('README.md') as f:
        return f.read()


setup(name='lootgenerator',
      version='0.1.0',
      description='Tabletop RPG loot generator',
      long_description=readme(),
      classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.7',
        'Topic :: Games/Entertainment',
        'Topic :: Games/Entertainment :: Board Games',
        'Topic :: Games/Entertainment :: Role-Playing',
        'Topic :: Utilities'
      ],
      keywords='rpg loot',
      url='http://github.com/tengro/lootgenerator',
      author='Andrew Liashchuk',
      author_email='tengro@gmail.com',
      license='MIT',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False)
